# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-19 20:18
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rest_survey', '0004_otheranswer_surveyed'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='otheranswer',
            unique_together=set([('question', 'surveyed')]),
        ),
    ]



# unicode: utf-8
from rest_framework.reverse import reverse
from rest_framework import serializers
from common.serializers import LinkSerializer
from .utils import (
    reverse_list,
    build_fieldsets,
    build_schema,
    build_question_name,
)
from .models import (
    Survey,
    Section,
    Question,
    SurveyedObject,
    Answer,
    OtherAnswer,
)


class QuestionSerializer(LinkSerializer):
    """
    survey serializer
    """

    options = serializers.JSONField()

    def get_links(self, obj):
        request = self.context['request']
        links = {
            'self': reverse('question-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'survey': reverse('survey-detail',
                              kwargs={'pk': obj.survey_id}),

        }
        if obj.section:
            links['section'] = reverse('section-detail',
                                       kwargs={'pk': obj.section_id},
                                       request=request)

        if obj.parent:
            links['parent'] = reverse('question-detail',
                                      kwargs={'pk': obj.parent_id},
                                      request=request)
        return links

    class Meta:
        model = Question
        fields = (
            'id',
            'date_created',
            'date_updated',
            'created_by',
            'order',
            'survey',
            'section',
            'parent',
            'parent_value',
            'type',
            'text',
            'help',
            'options',
            'links',
        )


class SectionSerializer(LinkSerializer):
    """
    section serializer
    """
    def get_links(self, obj):
        request = self.context['request']
        links = {
            'self': reverse('section-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'survey': reverse('survey-detail',
                              kwargs={'pk': obj.survey_id},
                              request=request),
            'questions': reverse_list(request,
                                      'question-list',
                                      'section_id',
                                      obj.pk)
        }

        return links

    class Meta:
        model = Section
        fields = (
            'id',
            'date_created',
            'date_updated',
            'created_by',
            'order',
            'survey',
            'title',
            'links',
        )


class SurveySerializer(LinkSerializer):
    """
    survey serializer
    """

    survey_data = serializers.SerializerMethodField()

    def get_survey_data(self, obj):
        return {
            'fieldsets': build_fieldsets(obj),
            'schema': build_schema(obj)
        }

    def get_links(self, obj):
        """
        returns all the relevant links
        for the survey model in the serializer
        """

        request = self.context['request']
        return {
            'self': reverse('survey-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'sections': reverse_list(request,
                                     'section-list',
                                     'survey_id',
                                     obj.pk),
            'questions': reverse_list(request,
                                      'question-list',
                                      'survey_id',
                                      obj.pk),
            'surveyeds': reverse_list(request,
                                      'surveyedobject-list',
                                      'survey_id',
                                      obj.pk),
        }

    class Meta:
        model = Survey
        fields = (
            'id',
            'date_created',
            'date_updated',
            'created_by',
            'name',
            'active',
            'survey_data',
            'links',
        )


class SurveyedObjectSerializer(LinkSerializer):
    """
    survey serializer
    """

    survey_data = serializers.SerializerMethodField()

    def get_survey_data(self, obj):
        return {build_question_name(a.question): a.value
                for a in obj.answers.all()}

    def get_links(self, obj):
        request = self.context['request']
        links = {
            'self': reverse('surveyedobject-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'answers': reverse_list(request,
                                    'question-list',
                                    'surveyedobject_id',
                                    obj.pk),
            'survey': reverse('survey-detail',
                              kwargs={'pk': obj.survey_id},
                              request=request)
        }
        return links

    class Meta:
        model = SurveyedObject
        fields = (
            'id',
            'content_type',
            'object_id',
            'links',
            'survey_data'
        )


class AnswerSerializer(LinkSerializer):
    """
    survey serializer
    """
    def get_links(self, obj):
        request = self.context['request']
        links = {
            'self': reverse('answer-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'surveyed': reverse('surveyedobject-detail',
                                kwargs={'pk': obj.surveyed_id},
                                request=request),
            'question': reverse('question-detail',
                                kwargs={'pk': obj.question_id},
                                request=request),
        }
        return links

    class Meta:
        model = Answer
        fields = (
            'id',
            'surveyed',
            'question',
            'value',
            'links',
        )


class OtherAnswerSerializer(LinkSerializer):
    """
    survey serializer
    """
    def get_links(self, obj):
        request = self.context['request']
        return {
            'self': reverse('otheranswer-detail',
                            kwargs={'pk': obj.pk},
                            request=request),
            'surveyed': reverse('surveyedobject-detail',
                                kwargs={'pk': obj.surveyed_id},
                                request=request),
            'question': reverse('question-detail',
                                kwargs={'pk': obj.question_id},
                                request=request)
        }

    class Meta:

        model = OtherAnswer
        fields = (
            'id',
            'question',
            'value',
            'surveyed',
            'links',
        )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-15 16:35
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields
import mptt.fields
import rest_survey.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Answer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('value', models.CharField(max_length=256, verbose_name='Value')),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Answer',
                'verbose_name_plural': 'Answers',
            },
        ),
        migrations.CreateModel(
            name='OtherAnswer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('value', models.CharField(max_length=256, verbose_name='Value')),
            ],
        ),
        migrations.CreateModel(
            name='Question',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False)),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('parent_value', models.CharField(help_text='Value that enables the user to answer the children questions.', max_length=256, null=True, verbose_name='Parent Value')),
                ('type', models.CharField(choices=[('Radio', 'Radio'), ('Checkboxes', 'Checkboxes'), ('Select', 'Select'), ('Text', 'Text'), ('TextArea', 'TextArea'), ('Number', 'Number'), ('Date', 'Date'), ('DateTime', 'DateTime'), ('List', 'List'), ('Grid', 'Grid')], default='Text', max_length=32, verbose_name='Question Type')),
                ('text', models.CharField(max_length=512, verbose_name='Text')),
                ('help', models.CharField(max_length=512, null=True, verbose_name='Help')),
                ('validators', jsonfield.fields.JSONField(default=rest_survey.models.default_validators, help_text='Validators to be applied at front-end level.', verbose_name='Validators')),
                ('other_token', models.CharField(max_length=16, null=True, verbose_name='Other Token')),
                ('options', jsonfield.fields.JSONField(help_text='Options to be used, if the field is SELECT, RADIO, CHECKBOX or GRID.', null=True, verbose_name='Options')),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('parent', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='children', to='rest_survey.Question')),
            ],
            options={
                'verbose_name': 'Question',
                'verbose_name_plural': 'Questions',
            },
        ),
        migrations.CreateModel(
            name='Section',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('order', models.PositiveIntegerField(db_index=True, editable=False)),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('title', models.CharField(max_length=128, verbose_name='Title')),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'ordering': ('survey', 'order'),
                'verbose_name': 'Section',
                'verbose_name_plural': 'Sections',
            },
        ),
        migrations.CreateModel(
            name='Survey',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('name', models.CharField(max_length=128, verbose_name='Name')),
                ('active', models.BooleanField(default=True, verbose_name='Active?')),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
            ],
            options={
                'verbose_name': 'Survey',
                'verbose_name_plural': 'Surveys',
            },
        ),
        migrations.CreateModel(
            name='SurveyedObject',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name='Date Created')),
                ('date_updated', models.DateTimeField(auto_now=True, verbose_name='Date Updated')),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='User')),
                ('survey', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='surveyeds', to='rest_survey.Survey')),
            ],
            options={
                'verbose_name': 'Surveyed Object',
                'verbose_name_plural': 'Surveyed Objects',
            },
        ),
        migrations.AddField(
            model_name='section',
            name='survey',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='sections', to='rest_survey.Survey'),
        ),
        migrations.AddField(
            model_name='question',
            name='section',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='rest_survey.Section'),
        ),
        migrations.AddField(
            model_name='question',
            name='survey',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='questions', to='rest_survey.Survey'),
        ),
        migrations.AddField(
            model_name='otheranswer',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='others', to='rest_survey.Question'),
        ),
        migrations.AddField(
            model_name='answer',
            name='question',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='rest_survey.Question'),
        ),
        migrations.AddField(
            model_name='answer',
            name='surveyed',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='answers', to='rest_survey.SurveyedObject'),
        ),
        migrations.AlterUniqueTogether(
            name='otheranswer',
            unique_together=set([('question', 'value')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-19 14:16
from __future__ import unicode_literals

from django.db import migrations
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('rest_survey', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='answer',
            name='value',
            field=jsonfield.fields.JSONField(null=True, verbose_name='Value'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-07-19 13:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('rest_survey', '0003_auto_20170619_0928'),
    ]

    operations = [
        migrations.AddField(
            model_name='otheranswer',
            name='surveyed',
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name='others',
                to='rest_survey.SurveyedObject'),
            preserve_default=False,
        ),
    ]



# coding: utf-8
from django.utils.translation import ugettext_lazy as _


QUESTION_TYPE_RADIO = u'Radio'
QUESTION_TYPE_CHECKBOXES = u'Checkboxes'
QUESTION_TYPE_SELECT = u'Select'
QUESTION_TYPE_TEXT = u'Text'
QUESTION_TYPE_TEXTAREA = u'TextArea'
QUESTION_TYPE_NUMBER = u'Number'
QUESTION_TYPE_DATE = u'Date'
QUESTION_TYPE_DATETIME = u'DateTime'
QUESTION_TYPE_LIST = u'List'
QUESTION_TYPE_GRID = u'Grid'

QUESTION_TYPE_CHOICES = (
    (QUESTION_TYPE_RADIO, _(u'Radio')),
    (QUESTION_TYPE_CHECKBOXES, _(u'Checkboxes')),
    (QUESTION_TYPE_SELECT, _(u'Select')),
    (QUESTION_TYPE_TEXT, _(u'Text')),
    (QUESTION_TYPE_TEXTAREA, _(u'TextArea')),
    (QUESTION_TYPE_NUMBER, _(u'Number')),
    (QUESTION_TYPE_DATE, _(u'Date')),
    (QUESTION_TYPE_DATETIME, _(u'DateTime')),
    (QUESTION_TYPE_LIST, _(u'List')),
    (QUESTION_TYPE_GRID, _(u'Grid')),
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.2 on 2017-06-19 14:28
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rest_survey', '0002_auto_20170619_0916'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='answer',
            unique_together=set([('surveyed', 'question')]),
        ),
    ]



# coding: utf-8
from rest_framework.reverse import reverse

from .choices import (
    QUESTION_TYPE_RADIO,
    QUESTION_TYPE_SELECT,
    QUESTION_TYPE_CHECKBOXES,
    QUESTION_TYPE_GRID
)


def answer_question(survey,
                    content_type_id,
                    object_id,
                    question_name,
                    value):
    """
    Answer a question for a single survey taken.

    this method should take care of it all.
    """
    from .models import (
        Answer,
        OtherAnswer,
        SurveyedObject,
    )
    try:
        new_survey = False
        surveyed = SurveyedObject.objects.get(
            content_type_id=content_type_id,
            object_id=object_id,
            survey_id=survey.pk
        )
    except SurveyedObject.DoesNotExist:
        new_survey = True
        surveyed = SurveyedObject(
            content_type_id=content_type_id,
            object_id=object_id,
            survey_id=survey.pk
        )

    question = get_question_by_name(question_name)
    if not question:
        raise ValueError('question does not exist')

    if question.survey_id != survey.pk:
        raise ValueError('question does not belong to this questionary')

    # possibilities
    # new survey and new question
    # old survey and new question
    # old survey and old question
    # this determines what is the case and updates/creates the answer value
    if not new_survey:
        # old survey
        try:
            # old question
            answer = Answer.objects.get(
                surveyed_id=surveyed.pk,
                question_id=question.pk
            )
        except Answer.DoesNotExist:
            # new question
            answer = Answer(
                surveyed=surveyed,
                question=question
            )
    else:
        # new survey and new question
        surveyed.save()
        answer = Answer(
            surveyed=surveyed,
            question=question
        )

    if value is None:
        answer.value = value
        answer.save()
        return answer

    answer.value = value

    if question.other_token:
        if question.other_token in value:

            question_options = question.options['values']
            question_options_upper = set([q.upper() for q in question_options])
            question_options_upper.add(question.other_token.upper())
            upper_value = [v.upper() if v else None for v in value]

            if set(upper_value).difference(question_options_upper):
                other_value = set(upper_value).difference(question_options_upper).pop()
                if other_value:
                    create_or_update_other_answer(surveyed, question, other_value)
                upper_value.remove(other_value)
                question_options.append(question.other_token)

            else:
                upper_value.remove(question.other_token.upper())
                delete_other_answer(surveyed, question)

            answer.value = get_options_in_upper(question_options, upper_value)

        else:
            delete_other_answer(surveyed, question)

    answer.save()
    return answer


def create_or_update_other_answer(surveyed, question, value):
    from .models import OtherAnswer
    try:
        other_answer = OtherAnswer.objects.get(question=question, surveyed=surveyed)
        other_answer.value = value
        other_answer.save()
    except OtherAnswer.DoesNotExist:
        OtherAnswer.objects.create(
            surveyed=surveyed,
            question=question,
            value=value.upper()
        )


def delete_other_answer(surveyed, question):
    from .models import OtherAnswer
    try:
        other_answer = OtherAnswer.objects.get(question=question, surveyed=surveyed)
        other_answer.delete()
    except OtherAnswer.DoesNotExist:
        return


def get_options_in_upper(options, upper):
    result = []
    for opt in options:
        if opt.upper() in upper:
            result.append(opt)

    return result


def reverse_list(request, view_name, field, obj):

    """
    reverses a list name
    """
    return reverse(view_name, request=request) + '?{0}={1}'.format(field, obj)


def build_survey_data(survey):
    """
    From a survey, build survey
    data.

    survey_data is a dict with all
    the options, ready for rendering
    """
    return {
        'schema': build_schema(survey),
        'fieldsets': build_fieldsets(survey)
    }


def build_answer(answer):

    return {build_question_name(answer.question): answer.value}


def build_answers(surveyed):

    answer_data = {}
    for a in surveyed.answers:
        answer_data.update(build_answer(a))
    return answer_data


def get_question_by_name(question_name):

    from .models import Question
    if question_name.startswith('q'):
        return Question.objects.get(id=int(question_name[1:]))

    return None


def build_question_name(question):

    return 'q' + str(question.id)


def build_fieldset_name(section):
    return 'f' + str(section.id)


def build_question(question):
    question_data = {}
    question_name = build_question_name(question)
    question_data[question_name] = {
        'name': question_name,
        'title': question.text,
        'help': question.help,
        'type': question.type,
        'options': question.options
    }
    if question.type in (QUESTION_TYPE_RADIO,
                         QUESTION_TYPE_SELECT,
                         QUESTION_TYPE_CHECKBOXES):

        question_data[question_name]['options'] = question.options['values']
        if question.other_token:
            question_data[question_name]['options'].append(question.other_token)
            question_data[question_name]['otherToken'] = question.other_token

    if question.type == QUESTION_TYPE_GRID:
        question_data[question_name]['columns'] = question.options.get('columns')
        question_data[question_name]['rows'] = question.options.get('rows')

    if question.validators:
        question_data[question_name]['validators'] = question.validators

    if question.parent:
        question_data[question_name]['parent'] = build_question_name(question.parent)
        question_data[question_name]['parent_value'] = question.parent_value

    return question_data


def build_schema(survey):
    schema = {}
    questions = survey.questions.all()
    for q in questions:
        schema.update(build_question(q))
    return schema


def build_fieldsets(survey):
    fieldsets = []
    sections = survey.sections.all()
    for section in sections:
        fieldsets.append(
            {
                'legend': section.title,
                'fields': [build_question_name(q) for q in section.questions.all().order_by('id')],
                'name': build_fieldset_name(section)
            }
        )
    return fieldsets



# -*- coding: utf-8 -*-
from django.conf.urls import include, url
from rest_framework.routers import DefaultRouter
from .viewsets import (
    SurveyViewSet,
    SectionViewSet,
    QuestionViewSet,
    SurveyedObjectViewSet,
    AnswerViewSet,
    OtherAnswerViewSet,
)

router = DefaultRouter(trailing_slash=False)
router.register(r'surveys', SurveyViewSet)
router.register(r'sections', SectionViewSet)
router.register(r'questions', QuestionViewSet)
router.register(r'surveyeds', SurveyedObjectViewSet)
router.register(r'answers', AnswerViewSet)
router.register(r'other-answers', OtherAnswerViewSet)

urlpatterns = [
    url(r'^api/', include(router.urls)),
]
